import type { MpSdk } from '..';
import * as THREE from 'three';
import type React from 'react';
export interface MatterportFocusCameraProps {
    /**
     * Target to look at at the end of the transition
     */
    target?: THREE.Vector3 | THREE.Vector3Tuple | THREE.Object3D;
    /**
     * Preferred position to look at the target FROM
     * - In certain modes, the final position may be constrained to a valid viewpoint
     *   ex: in Mode.INSIDE, restrict the final placement to a valid sweep position.
     *
     * defaults to current camera position.
     */
    from?: THREE.Vector3 | THREE.Vector3Tuple;
    /**
     * transition style
     *
     * defaults to 'transition.fly'
     */
    transition?: MpSdk.Mode.TransitionType;
    /**
     * View mode to finish the transition in
     */
    mode?: MpSdk.Mode.Mode;
    /**
     * if true, blocks navigation events normally triggered via mesh.onClick
     *
     * defaults to 'false'
     */
    blockNavigation?: boolean;
}
/**
 * A wrapper around matterport viewer which provides a lightwight way to control the view focalpoint
 * while using the matterport camera controls.
 */
export declare const MatterportFocusCamera: React.FC<MatterportFocusCameraProps>;
