import * as THREE from 'three';
import { useEffect, useState } from 'react';
import { useMatterportSdk } from '../hooks';
export const MatterportFocusCamera = ({ blockNavigation, target, from, transition, mode, }) => {
    const mpSdk = useMatterportSdk();
    const [playing, setPlaying] = useState();
    useEffect(() => {
        const sub = mpSdk === null || mpSdk === void 0 ? void 0 : mpSdk.App.state.subscribe(s => setPlaying(s.phase === mpSdk.App.Phase.PLAYING));
        return sub === null || sub === void 0 ? void 0 : sub.cancel;
    }, [mpSdk]);
    useEffect(() => {
        mpSdk === null || mpSdk === void 0 ? void 0 : mpSdk.R3F.navigationToggle(!blockNavigation);
        mpSdk === null || mpSdk === void 0 ? void 0 : mpSdk.R3F.controlsToggle(!blockNavigation);
    }, [mpSdk, blockNavigation]);
    useEffect(() => {
        if (!target)
            return;
        if (!mpSdk)
            return;
        if (!playing)
            return;
        mpSdk === null || mpSdk === void 0 ? void 0 : mpSdk.R3F.focus(toTarget(target), {
            transition: transition === 'transition.instant' ? mpSdk === null || mpSdk === void 0 ? void 0 : mpSdk.Mode.TransitionType.INSTANT : mpSdk === null || mpSdk === void 0 ? void 0 : mpSdk.Mode.TransitionType.FLY,
            from: from ? toVec3(from) : undefined,
            mode: (mode !== (mpSdk === null || mpSdk === void 0 ? void 0 : mpSdk.Mode.Mode.OUTSIDE) && mode !== (mpSdk === null || mpSdk === void 0 ? void 0 : mpSdk.Mode.Mode.TRANSITIONING)) ? mode : undefined,
        });
    }, [target, from, transition, mode, playing, mpSdk]);
    return null;
};
const toVec3 = (() => {
    const tempVec = new THREE.Vector3();
    return (input) => {
        if (Array.isArray(input) && input.length === 3) {
            tempVec.fromArray(input);
        }
        else if (input instanceof THREE.Vector3) {
            tempVec.copy(input);
        }
        return tempVec;
    };
})();
const toTarget = (() => {
    const tempVec = new THREE.Vector3();
    const tempBox = new THREE.Box3();
    return (input) => {
        if (input instanceof THREE.Object3D) {
            return tempBox.setFromObject(input);
        }
        else {
            return tempVec.copy(toVec3(input));
        }
    };
})();
