import { useMatterportSdk } from '../hooks';
import { useEffect } from 'react';
import { useObjectR3FEvents } from '../hooks/events';
import { useMatterportScene } from '../hooks/scene';
const registeredTextures = new Set();
const noop = () => {
};
export const MatterportPointer = (props) => {
    const sdk = useMatterportSdk();
    const { visible, textureSrc } = props;
    useEffect(() => {
        if (sdk && visible === false) {
            sdk.Pointer.setVisible(false);
            return () => {
                sdk.Pointer.setVisible(true);
            };
        }
        return noop;
    }, [sdk, visible]);
    useEffect(() => {
        if (sdk && textureSrc) {
            const textureId = `r3f:${textureSrc}`;
            (registeredTextures.has(textureId) ? Promise.resolve() : sdk.Asset.registerTexture(textureId, textureSrc))
                .then(() => sdk.Pointer.editTexture(textureId));
            registeredTextures.add(textureId);
            return () => {
                sdk.Pointer.resetTexture();
            };
        }
        return noop;
    }, [sdk, textureSrc]);
    const mpScene = useMatterportScene();
    useObjectR3FEvents(mpScene, props);
    return null;
};
export function clearRegisteredTextures() {
    registeredTextures.clear();
}
