import React, { type ComponentPropsWithRef } from 'react';
import '@matterport/webcomponent';
import type { MpSdk } from '..';
declare global {
    namespace JSX {
        interface IntrinsicElements {
            'matterport-viewer': MatterportViewer & ComponentPropsWithRef<any>;
        }
    }
}
interface MatterportViewerProps extends ComponentPropsWithRef<any> {
    m: string;
    applicationKey: string;
    assetBase: string;
    /**
     * the SDK is connected, the app enters the Starting state, and ready to use.
     * {@link app.phase|https://matterport.github.io/showcase-sdk/docs/reference/current/enums/app.phase.html#starting}
     *
     * @since — v0.2.2
     */
    onConnected?: (sdk: MpSdk) => void;
    /**
     * the SDK is connected and ready to use, and the camera is ready to start accepting commands
     * (ie, any intro fly-in has completed and the camera has stopped moving).
     * {@link app.phase|https://matterport.github.io/showcase-sdk/docs/reference/current/enums/app.phase.html#playing}
     *
     * @since — v0.2.2
     */
    onPlaying?: (sdk: MpSdk) => void;
    /**
     * Called when an error is thrown during asset loading or mpSdk {@link connect|https://matterport.github.io/showcase-sdk/docs/reference/current/interfaces/mp_sdk.html#connect}.
     *
     * 'LoadFailureError' Failed to load the matterport static assets at the configured `assetBase` path.
     *
     * 'KeyReferrerMismatchError' Unable to validate that the hosting domain is in the allowed list for the current Application Key. Verify that the domain is in the Developer Tools Settings on the Matterport Account Settings.
     *
     * 'InvalidProviderError' Unable to validate that the ConnectOptions.provider has sufficient access to the current space.
     *
     * @since — v0.2.3
     */
    onError?: (error?: Error) => void;
    /**
     * use `onConnected` instead, `onReady` will be removed in a future release.
     * @deprecated — v0.2.2
     */
    onReady?: (sdk: MpSdk) => void;
}
/**
 * Light react wrapper around the @matterport/webcomponent which inserts children @react-three/fiber components inside the matterport space.
 * Most props are ultimately forwarded to the @matterport/webcomponent. For full documentation of the {@link webcomponent|https://matterport.github.io/showcase-sdk/sdkbundle_home.html#web-component-method}
 *
 * @param props @see MatterportViewerProps {@link MatterportViewerProps}
 */
declare const MatterportViewerR3F: React.ForwardRefExoticComponent<Pick<MatterportViewerProps, keyof MatterportViewerProps> & React.RefAttributes<MatterportViewer>>;
export { MatterportViewerR3F as MatterportViewer, type MatterportViewerProps, };
export default MatterportViewerR3F;
