import React, { useLayoutEffect, useRef, useState, useEffect, } from 'react';
import '@matterport/webcomponent';
import { R3FSDKAdapter } from '../R3FSDKAdapter';
import { ObservableValue } from '../ObservableValue';
const MatterportViewerR3F = React.forwardRef((props, ref) => {
    const [childrenObservable] = useState(() => new ObservableValue(null));
    const [error, setError] = useState(null);
    const viewerRef = useForwardedRef(ref);
    useLayoutEffect(() => {
        const el = viewerRef.current;
        const onConnected = (e) => {
            var _a, _b, _c;
            const sdk = (_a = e.detail) === null || _a === void 0 ? void 0 : _a.mpSdk;
            const r3fAdapter = new R3FSDKAdapter(sdk);
            r3fAdapter.register(React.createElement(R3FContainer, { observable: childrenObservable }));
            (_b = props.onConnected) === null || _b === void 0 ? void 0 : _b.call(props, sdk);
            (_c = props.onReady) === null || _c === void 0 ? void 0 : _c.call(props, sdk);
        };
        const onPlaying = (e) => {
            var _a, _b;
            const sdk = (_a = e.detail) === null || _a === void 0 ? void 0 : _a.mpSdk;
            (_b = props.onPlaying) === null || _b === void 0 ? void 0 : _b.call(props, sdk);
        };
        const onError = (e) => {
            var _a, _b, _c;
            setError((_a = e.detail) === null || _a === void 0 ? void 0 : _a.error);
            (_b = props.onError) === null || _b === void 0 ? void 0 : _b.call(props, (_c = e.detail) === null || _c === void 0 ? void 0 : _c.error);
        };
        setError(null);
        el === null || el === void 0 ? void 0 : el.addEventListener('mpSdkPlaying', onPlaying);
        el === null || el === void 0 ? void 0 : el.addEventListener('mpSdkConnected', onConnected);
        el === null || el === void 0 ? void 0 : el.addEventListener('mpViewerError', onError);
        return () => {
            el === null || el === void 0 ? void 0 : el.removeEventListener('mpSdkConnected', onConnected);
            el === null || el === void 0 ? void 0 : el.removeEventListener('mpSdkPlaying', onPlaying);
            el === null || el === void 0 ? void 0 : el.removeEventListener('mpViewerError', onError);
        };
    }, [viewerRef, props, childrenObservable]);
    useEffect(() => {
        childrenObservable.value = props.children;
    }, [props.children, childrenObservable]);
    if (error) {
        throw error;
    }
    return (React.createElement("matterport-viewer", Object.assign({}, props, { ref: viewerRef, "asset-base": props.assetBase, "application-key": props.applicationKey, "connectSdk-type": 'npm.r3f', children: null })));
});
const R3FContainer = ({ observable }) => {
    const [children, setChildren] = useState(observable === null || observable === void 0 ? void 0 : observable.value);
    useEffect(() => {
        const sub = observable.onChanged(setChildren);
        return () => sub.cancel();
    }, [observable]);
    return React.createElement(React.Fragment, null, children);
};
function useForwardedRef(ref) {
    const innerRef = useRef(null);
    useEffect(() => {
        if (!ref)
            return;
        if (typeof ref === 'function') {
            ref(innerRef.current);
        }
        else {
            ref.current = innerRef.current;
        }
    });
    return innerRef;
}
export { MatterportViewerR3F as MatterportViewer, };
export default MatterportViewerR3F;
