import type { Object3D } from 'three';
import type { ISubscription } from '@matterport/webcomponent';
import type { render as r3fRender } from '@react-three/fiber';
/**
 * Initializes tracking of meshes for pointer event registration.
 */
export declare function initEvents(r3fStore: ReturnType<typeof r3fRender>, registerMeshEvents: (obj: Object3D) => ISubscription): void;
/**
 * Internally, R3F looks at each raycastable object's props when reconciling, and
 * if there are any event handler props present it adds that object to the plain
 * `rootState.internal.interaction` array. That array is normally iterated later on
 * when R3F performs raycasting.
 *
 * But since we're taking over all events, we need to know when objects are added/removed
 * from that `interaction` array so we can register/unregister them from Showcase's
 * event system. This function does some tricky overriding to track those changes.
 *
 * Going forward, we should add real support in R3F for observing the set of interaction
 * objects and contribute it upstream.
 */
export declare function observeInteractionArray(r3fInternal: {
    interaction: Object3D[];
}, onObjectAdded: (object: Object3D) => void, onObjectRemoved: (object: Object3D) => void): void;
