export function initEvents(r3fStore, registerMeshEvents) {
    const subscriptions = new Map();
    function onObjectAdded(obj) {
        if (subscriptions.has(obj)) {
            throw new Error('double subscription');
        }
        const sub = registerMeshEvents(obj);
        subscriptions.set(obj, sub);
    }
    function onObjectRemoved(obj) {
        const sub = subscriptions.get(obj);
        if (!sub) {
            throw new Error('no subscription for object');
        }
        sub.cancel();
        subscriptions.delete(obj);
    }
    observeInteractionArray(r3fStore.getState().internal, onObjectAdded, onObjectRemoved);
    r3fStore.subscribe((newState, oldState) => {
        if (newState.internal && newState.internal !== oldState.internal) {
            observeInteractionArray(newState.internal, onObjectAdded, onObjectRemoved);
        }
    });
}
export function observeInteractionArray(r3fInternal, onObjectAdded, onObjectRemoved) {
    let currentArray = r3fInternal.interaction;
    wrapArray(currentArray);
    Object.defineProperty(r3fInternal, 'interaction', {
        get() {
            return currentArray;
        },
        set(arr) {
            const oldSet = new Set(currentArray);
            const newSet = new Set(arr);
            for (const obj of oldSet) {
                if (!newSet.has(obj)) {
                    onObjectRemoved(obj);
                }
            }
            for (const obj of newSet) {
                if (!oldSet.has(obj)) {
                    onObjectAdded(obj);
                }
            }
            wrapArray(arr);
            currentArray = arr;
        },
    });
    function wrapArray(arr) {
        arr.push = function (...objects) {
            objects.forEach(onObjectAdded);
            return Array.prototype.push.apply(arr, objects);
        };
        arr.splice = function (start, deleteCount = Infinity) {
            arr.slice(start, start + deleteCount).forEach(onObjectRemoved);
            return Array.prototype.splice.call(arr, start, deleteCount);
        };
    }
}
