import { useEffect, useRef, } from 'react';
const knownEventProps = new Set([
    'onClick',
    'onContextMenu',
    'onDoubleClick',
    'onPointerUp',
    'onPointerDown',
    'onPointerOver',
    'onPointerOut',
    'onPointerEnter',
    'onPointerLeave',
    'onPointerMove',
    'onPointerMissed',
    'onPointerCancel',
    'onWheel',
]);
const targetObjectStates = new WeakMap();
let componentIndex = 0;
export function useObjectR3FEvents(target, handlers) {
    const ref = useRef();
    if (!ref.current) {
        ref.current = `mp-${componentIndex++}`;
    }
    const componentId = ref.current;
    let r3fState;
    let targetObjState;
    let instanceHandlers;
    if (target) {
        r3fState = target.__r3f || (target.__r3f = {
            handlers: {},
            eventCount: 0,
        });
        targetObjState = targetObjectStates.get(target);
        if (!targetObjState) {
            targetObjectStates.set(target, targetObjState = {
                eventCounts: {},
                instanceHandlers: {},
            });
        }
        instanceHandlers = targetObjState.instanceHandlers[componentId] || (targetObjState.instanceHandlers[componentId] = {});
        let eventProp;
        for (eventProp in instanceHandlers) {
            if (!handlers[eventProp]) {
                delete instanceHandlers[eventProp];
                if (--targetObjState.eventCounts[eventProp] === 0) {
                    r3fState.eventCount--;
                    delete r3fState.handlers[eventProp];
                }
            }
            else {
                instanceHandlers[eventProp] = handlers[eventProp];
            }
        }
        for (eventProp in handlers) {
            if (!instanceHandlers[eventProp] && knownEventProps.has(eventProp)) {
                instanceHandlers[eventProp] = handlers[eventProp];
                const prevCount = targetObjState.eventCounts[eventProp] || 0;
                targetObjState.eventCounts[eventProp] = prevCount + 1;
                if (prevCount === 0) {
                    r3fState.eventCount++;
                    const scopedEventProp = eventProp;
                    r3fState.handlers[eventProp] = (event) => {
                        var _a, _b;
                        for (const id in targetObjState === null || targetObjState === void 0 ? void 0 : targetObjState.instanceHandlers) {
                            (_b = targetObjState === null || targetObjState === void 0 ? void 0 : (_a = targetObjState.instanceHandlers[id])[scopedEventProp]) === null || _b === void 0 ? void 0 : _b.call(_a, event);
                        }
                    };
                }
            }
        }
    }
    useEffect(() => {
        return () => {
            if (instanceHandlers && targetObjState && r3fState) {
                let prop;
                for (prop in instanceHandlers) {
                    delete instanceHandlers[prop];
                    if (--targetObjState.eventCounts[prop] === 0) {
                        r3fState.eventCount--;
                        delete r3fState.handlers[prop];
                    }
                }
            }
        };
    }, [targetObjState, r3fState, instanceHandlers]);
}
